/*

  - mkzone 
    Creates a zone and a revers lookup file for bind with automatically
    generated names, from a class C net and an optional IP range.
    If invoked without parameters, the 192.168.0.0/24 range is created.

  - Copyright 2001 by Jarle (jgaa) Aase, http://www.jgaa.com
    Released under Gnu Public Lisence. See the file COPYING

*/


#include <iostream>
#include <fstream>
#include <stdio.h>     
#include <getopt.h>
#include <stdlib.h>
  

const char *domain_name = "local.lan";
const char *class_c_net = "192.168.0.";
const char *zone_file = "zonefile";
const char *rev_file = "revfile";
int from_ip = 1;
int to_ip = 254;
int do_header = 1;


using namespace std;

void usage()
{
  cout << "mkzone [options]" << endl
       << "-n  --domain_name=name         Domain name" << endl
       << "-c  --class-c-net=aaa.bbb.ccc. Class C net" << endl
       << "-f  --from-ip=#                From IP number" << endl
       << "-t  --to-ip=#                  To IP number" << endl
       << "-z  --zone-file=path           Zone file" << endl
       << "-r  --rev-file=path            Rev dns file" << endl
       << "    --no-header                Supress header" << endl
       << "-h  --help                     Show this help" << endl;
  exit(-1);
}

int main(int argc, char *argv[])
{
  static const option options[] = {
    {"domain-name", required_argument, NULL, 'n' },
    {"class-c-net", required_argument, NULL, 'c' },
    {"from-ip", required_argument, NULL, 'f' },
    {"to-ip", required_argument, NULL, 't' },
    {"zone-file", required_argument, NULL, 'z' },
    {"rev-file", required_argument, NULL, 'r' },
    {"no-header", no_argument, &do_header, 0},
    {"help", no_argument, NULL, 'h'}
  };

  int ch;

  while((ch = getopt_long(argc, argv, "hn:c:f:t:z:r:", options, NULL)) >= 0) {
    switch(ch) {
    case 'n':
      domain_name = optarg;
      break;
    case 'c':
      class_c_net = optarg;
      break;
    case 'f':
      from_ip = atoi(optarg);
      break;
    case 't':
      to_ip = atoi(optarg);
      break;
    case 'z':
      zone_file = optarg;
      break;
    case 'r':
      rev_file = optarg;
      break;
    case 'h':
    case '?':
      usage();
      break;
    }
  }
    
  ofstream zone(zone_file), rev(rev_file);

  if (!zone){
    cerr << "Failed to open \"" << zone_file << "\" for write!" << endl;
    return -1;
  }
    
  if (!rev){
    cerr << "Failed to open \"" << rev_file << "\" for write!" << endl;
    return -1;
  }


  // Calculate rev-order
  int i1 = 0, i2 = 0, i3= 0;
  if (sscanf(class_c_net, "%d.%d.%d.", &i1, &i2, &i3) != 3) {
    cerr << "Invalid class C net spec!" << class_c_net << endl;
    return -1;
  }
  char class_c_rev[64];
  sprintf(class_c_rev, "%d.%d.%d.in-addr.arpa.", i3, i2, i1);

  if (do_header) {
    zone << "; Zone file for " << domain_name << " generated by mkzone" << endl
	 << domain_name << ". IN SOA ns." << domain_name << ". hostmaster." << domain_name << " (" << endl
	 << "\t1\t; serial" << endl
	 << "\t10800\t; Refresh after 3 hours" << endl
	 << "\t3600\t; Retry after 1 hous" << endl
	 << "\t604800\t; Expire after 1 week" << endl
	 << "\t86400 )\t; Minimum TTL of 1 day" << endl
	 << endl
	 << "@\t\tIN NS ns." << domain_name << "." << endl
	 << endl;

    rev << "; Zone file for " << domain_name << " generated by mkzone" << endl
	<< class_c_rev << " IN SOA ns." << domain_name << ". hostmaster." << domain_name << " (" << endl
	<< "\t1\t; serial" << endl
	<< "\t10800\t; Refresh after 3 hours" << endl
	<< "\t3600\t; Retry after 1 hous" << endl
	<< "\t604800\t; Expire after 1 week" << endl
	<< "\t86400 )\t; Minimum TTL of 1 day" << endl
	<< endl
	<< "@\t\tIN NS\tns." << domain_name << "." << endl
	<< endl;
    cout << "Note: You must manually add ns." << domain_name << ". to the zonefile." << endl;
  }

  for(int i = from_ip; i <= to_ip; i++) {
    zone << class_c_net << i << "." << domain_name << ".\tIN A\t" << class_c_net << i << endl;
    rev << i << "." << class_c_rev << "\t\tIN PTR\t" << class_c_net << i << "." << domain_name << "." << endl; 
  }


  return 0;
}


